/*
 * Copyright (C) 2006 The Android Open Source Project
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package com.xtremelabs.robolectric.shadows;

import java.util.List;
import java.util.Map;

import android.content.Context;
import android.database.Cursor;
import android.net.Uri;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Checkable;
import android.widget.ImageView;
import android.widget.ListAdapter;
import android.widget.SimpleAdapter;
import android.widget.TextView;
import android.widget.SimpleAdapter.ViewBinder;

import com.xtremelabs.robolectric.internal.Implementation;
import com.xtremelabs.robolectric.internal.Implements;

/**
 * The {@link SimpleAdapter} Shadow. It works similar to the real one This is
 * important if you don't want to use a {@link Cursor} for your information If
 * we didn't add the shadow to the {@link SimpleAdapter} the calls to add some
 * DataObserver it wouldn't work
 */
@Implements(SimpleAdapter.class)
public class ShadowSimpleAdapter extends ShadowBaseAdapter {

    private int mLayout;

    private LayoutInflater mInflater;

    private int mDropDownLayout;

    /**
     * A list of columns containing the data to bind to the UI. This field
     * should be made private, so it is hidden from the SDK. {@hide}
     */
    protected int[] mFrom;
    /**
     * A list of View ids representing the views to which the data must be
     * bound. This field should be made private, so it is hidden from the SDK.
     * {@hide}
     */
    protected int[] mTo;

    private ViewBinder mViewBinder;
    private String[] mOriginalFrom;

    /** The map with the actual data. */
    private List<? extends Map<String, ?>> mData;

    /**
     * Constructor.
     *
     * @param context
     *            The context where the ListView associated with this
     *            SimpleListItemFactory is running
     * @param layout
     *            resource identifier of a layout file that defines the views
     *            for this list item. The layout file should include at least
     *            those named views defined in "to"
     * @param data
     *            The List that contains the actual data. Each ListItem is a Map
     *            with a key value pair for all the values.
     * @param from
     *            A list of column names representing the data to bind to the
     *            UI. Can be null if the cursor is not available yet.
     * @param to
     *            The views that should display column in the "from" parameter.
     *            These should all be TextViews. The first N views in this list
     *            are given the values of the first N columns in the from
     *            parameter. Can be null if the cursor is not available yet.
     */
    public void __constructor__(final Context context,
                                final java.util.List<? extends java.util.Map<java.lang.String, ?>> data, final int layout,
                                final String[] from, final int[] to) {
        mLayout = mDropDownLayout = layout;
        mTo = to;
        mOriginalFrom = from;
        mData = data;
        mInflater = (LayoutInflater) context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    }

    /**
     * Returns the {@link ViewBinder} used to bind data to views.
     *
     * @return a ViewBinder or null if the binder does not exist
     *
     * @see #bindView(android.view.View, android.content.Context,
     *      android.database.Cursor)
     * @see #setViewBinder(com.xtremelabs.robolectric.shadows.ShadowSimpleCursorAdapter.ViewBinder)
     */
    @Implementation
    public ViewBinder getViewBinder() {
        return mViewBinder;
    }

    /**
     * Sets the binder used to bind data to views.
     *
     * @param viewBinder
     *            the binder used to bind data to views, can be null to remove
     *            the existing binder
     *
     * @see #bindView(android.view.View, android.content.Context,
     *      android.database.Cursor)
     * @see #getViewBinder()
     */
    @Implementation
    public void setViewBinder(final ViewBinder viewBinder) {
        mViewBinder = viewBinder;
    }

    /**
     * Called by bindView() to set the image for an ImageView but only if there
     * is no existing ViewBinder or if the existing ViewBinder cannot handle
     * binding to an ImageView.
     *
     * By default, the value will be treated as an image resource. If the value
     * cannot be used as an image resource, the value is used as an image Uri.
     *
     * Intended to be overridden by Adapters that need to filter strings
     * retrieved from the database.
     *
     * @param v
     *            ImageView to receive an image
     * @param value
     *            the value retrieved from the cursor
     */
    @Implementation
    public void setViewImage(final ImageView v, final String value) {
        try {
            v.setImageResource(Integer.parseInt(value));
        } catch (NumberFormatException nfe) {
            v.setImageURI(Uri.parse(value));
        }
    }

    @Implementation
    public void setViewImage(final ImageView v, final int value) {
        v.setImageResource(value);
    }

    /**
     * Called by bindView() to set the text for a TextView but only if there is
     * no existing ViewBinder or if the existing ViewBinder cannot handle
     * binding to an TextView.
     *
     * Intended to be overridden by Adapters that need to filter strings
     * retrieved from the database.
     *
     * @param v
     *            TextView to receive text
     * @param text
     *            the text to be set for the TextView
     */
    @Implementation
    public void setViewText(final TextView v, final String text) {
        v.setText(text);
    }

    /**
     * @see ListAdapter#getView(int, View, ViewGroup)
     */
    @Implementation
    public View getView(final int position, final View convertView, final ViewGroup parent) {
        View v;
        if (convertView == null) {
            v = mInflater.inflate(mLayout, parent, false);
        } else {
            v = convertView;
        }

        bindView(position, v);

        return v;
    }

    /**
     * This binds the data from the List to a certain view.
     * @param position
     *                  The position of the data list. The list item
     * @param view
     *              The view to inject the data                  
     */
    private void bindView(final int position, final View view) {
        final Map dataSet = mData.get(position);
        if (dataSet == null) {
            return;
        }

        final ViewBinder binder = mViewBinder;
        final String[] from = mOriginalFrom;
        final int[] to = mTo;
        final int count = to.length;

        for (int i = 0; i < count; i++) {
            final View v = view.findViewById(to[i]);
            if (v != null) {
                final Object data = dataSet.get(from[i]);
                String text = data == null ? "" : data.toString();
                if (text == null) {
                    text = "";
                }

                boolean bound = false;
                if (binder != null) {
                    bound = binder.setViewValue(v, data, text);
                }

                if (!bound) {
                    if (v instanceof Checkable) {
                        if (data instanceof Boolean) {
                            ((Checkable) v).setChecked((Boolean) data);
                        } else if (v instanceof TextView) {
                            // Note: keep the instanceof TextView check at the
                            // bottom of these
                            // ifs since a lot of views are TextViews (e.g.
                            // CheckBoxes).
                            setViewText((TextView) v, text);
                        } else {
                            throw new IllegalStateException(v.getClass().getName()
                                    + " should be bound to a Boolean, not a "
                                    + (data == null ? "<unknown type>" : data.getClass()));
                        }
                    } else if (v instanceof TextView) {
                        // Note: keep the instanceof TextView check at the
                        // bottom of these
                        // ifs since a lot of views are TextViews (e.g.
                        // CheckBoxes).
                        setViewText((TextView) v, text);
                    } else if (v instanceof ImageView) {
                        if (data instanceof Integer) {
                            setViewImage((ImageView) v, (Integer) data);
                        } else {
                            setViewImage((ImageView) v, text);
                        }
                    } else {
                        throw new IllegalStateException(v.getClass().getName() + " is not a "
                                + " view that can be bounds by this SimpleAdapter");
                    }
                }
            }
        }

    }

    /**
     * @see Adapter#getCount()
     */
    @Implementation
    public int getCount() {
        return mData.size();
    }

    /**
     * @see Adapter#getItem(int)
     */
    @Implementation
    public Object getItem(final int position) {
        return mData.get(position);
    }

    /**
     * Gets the DropDownView dependning int he position
     */
    @Implementation
    public View getDropDownView(final int position, final View convertView, final ViewGroup parent) {
        View v;
        if (convertView == null) {
            v = mInflater.inflate(mDropDownLayout, parent, false);
        } else {
            v = convertView;
        }

        bindView(position, v);
        return v;
    }

    /**
     * <p>
     * Sets the layout resource of the drop down views.
     * </p>
     *
     * @param dropDownLayout
     *            the layout resources used to create drop down views
     */
    @Implementation
    public void setDropDownViewResource(final int dropDownLayout) {
        mDropDownLayout = dropDownLayout;
    }

}